@@ -61,8 +61,10 @@ use ark_std::{
     vec::Vec,
     One, UniformRand,
 };
-use schnorr_pok::{SchnorrCommitment, SchnorrResponse};
-
+use dock_crypto_utils::serde_utils::*;
+use schnorr_pok::{error::SchnorrError, SchnorrCommitment, SchnorrResponse};
+use serde::{Deserialize, Serialize};
+use serde_with::serde_as;
 pub use serialization::*;
 
 /// Stateful protocol to prove knowledge of signature. The protocol randomizes the signature and executes 2 Schnorr
@@ -72,30 +74,44 @@ pub use serialization::*;
 /// challenge and post-challenge (`gen_proof`). Thus, several instances of the protocol can be used
 /// together where the pre-challenge phase of all protocols is used to create a combined challenge and then
 /// that challenge is used in post-challenge phase of all protocols.
-#[derive(Clone, PartialEq, Eq, Debug)]
+#[serde_as]
+#[derive(Clone, PartialEq, Eq, Debug, Serialize, Deserialize)]
 pub struct PoKOfSignatureG1Protocol<E: PairingEngine> {
+    #[serde_as(as = "AffineGroupBytes")]
     pub A_prime: E::G1Affine,
+    #[serde_as(as = "AffineGroupBytes")]
     pub A_bar: E::G1Affine,
+    #[serde_as(as = "AffineGroupBytes")]
     pub d: E::G1Affine,
-    /// For proving relation a_bar / d == a_prime^{-e} * h_0^r2
+    /// For proving relation `A_bar - d = A_prime * -e + h_0 * r2`
     pub sc_comm_1: SchnorrCommitment<E::G1Affine>,
+    #[serde_as(as = "[FieldBytes; 2]")]
     sc_wits_1: [E::Fr; 2],
-    /// For proving relation g1 * h1^m1 * h2^m2.... for all disclosed messages m_i == d^r3 * h_0^{-s_prime} * h1^-m1 * h2^-m2.... for all undisclosed messages m_i
+    /// For proving relation `g1 + h1*m1 + h2*m2 +.... + h_i*m_i` = `d*r3 + {h_0}*{-s_prime} + h1*{-m1} + h2*{-m2} + .... + h_j*m_j` for all disclosed messages `m_i` and for all undisclosed messages `m_j`
     pub sc_comm_2: SchnorrCommitment<E::G1Affine>,
+    #[serde_as(as = "Vec<FieldBytes>")]
     sc_wits_2: Vec<E::Fr>,
 }
 
 /// Proof of knowledge of the signature. It contains the randomized signature, commitment (Schnorr step 1)
 /// and response (Schnorr step 3) to both Schnorr protocols in `T_` and `sc_resp_`
-#[derive(Clone, PartialEq, Eq, Debug, CanonicalSerialize, CanonicalDeserialize)]
+#[serde_as]
+#[derive(
+    Clone, PartialEq, Eq, Debug, CanonicalSerialize, CanonicalDeserialize, Serialize, Deserialize,
+)]
 pub struct PoKOfSignatureG1Proof<E: PairingEngine> {
+    #[serde_as(as = "AffineGroupBytes")]
     pub A_prime: E::G1Affine,
+    #[serde_as(as = "AffineGroupBytes")]
     pub A_bar: E::G1Affine,
+    #[serde_as(as = "AffineGroupBytes")]
     pub d: E::G1Affine,
-    /// Proof of relation a_bar / d == a_prime^{-e} * h_0^r2
+    /// Proof of relation `A_bar - d = A_prime * -e + h_0 * r2`
+    #[serde_as(as = "AffineGroupBytes")]
     pub T1: E::G1Affine,
     pub sc_resp_1: SchnorrResponse<E::G1Affine>,
-    /// Proof of relation g1 * h1^m1 * h2^m2.... for all disclosed messages m_i == d^r3 * h_0^{-s_prime} * h1^-m1 * h2^-m2.... for all undisclosed messages m_i
+    /// Proof of relation `g1 + h1*m1 + h2*m2 +.... + h_i*m_i` = `d*r3 + {h_0}*{-s_prime} + h1*{-m1} + h2*{-m2} + .... + h_j*m_j` for all disclosed messages `m_i` and for all undisclosed messages `m_j`
+    #[serde_as(as = "AffineGroupBytes")]
     pub T2: E::G1Affine,
     pub sc_resp_2: SchnorrResponse<E::G1Affine>,
 }
@@ -162,18 +178,18 @@ where
         // s' = s - r2*r3
         let s_prime = signature.s - (r2 * r3);
 
-        // For proving relation a_bar / d == a_prime^{-e} * h_0^r2
+        // For proving relation A_bar - d == A'*{-e} + h_0*r2
         let bases_1 = [A_prime_affine, params.h_0.clone()];
         let wits_1 = [-signature.e, r2];
         let sc_comm_1 = SchnorrCommitment::new(&bases_1, vec![E::Fr::rand(rng), E::Fr::rand(rng)]);
 
-        // For proving relation g1 * h1^m1 * h2^m2.... for all disclosed messages m_i == d^r3 * h_0^{-s_prime} * h1^-m1 * h2^-m2.... for all undisclosed messages m_j
+        // For proving relation `g1 + h1*m1 + h2*m2 +.... + h_i*m_i` = `d*r3 + {h_0}*{-s_prime} + h1*{-m1} + h2*{-m2} + .... + h_j*m_j` for all disclosed messages `m_i` and for all undisclosed messages `m_j`
         // Usually the number of disclosed messages is much less than the number of undisclosed messages, its better to avoid negations in hidden messages and do
         // them in revealed messages. So transform the relation
-        // g1 * h1^m1 * h2^m2.... * h_i^m_i for disclosed messages m_i = d^r3 * h_0^{-s_prime} * h1^-m1 * h2^-m2.... * h_j^-m_j for all undisclosed messages m_j
+        // `g1 + h1*m1 + h2*m2 +.... + h_i*m_i` = `d*r3 + {h_0}*{-s_prime} + h1*{-m1} + h2*{-m2} + .... + h_j*m_j` for all disclosed messages `m_i` and for all undisclosed messages `m_j`
         // into
-        // d^{-r3} * h_0^s_prime * h1^m1 * h2^m2.... * h_j^m_j = g1 * h1^-m1 * h2^-m2.... * h_i^-m_i. Moreover g1 * h1^-m1 * h2^-m2.... * h_i^-m_i is public
-        // and can be efficiently computed as (g1 * h1^m1 * h2^m2.... * h_i^m_i)^-1 and inverse in elliptic group is a point negation which is very cheap
+        // d*{-r3} + h_0*s_prime + h1*m1 + h2*m2.... + h_j*m_j = -g1 + h1*{-m1} + h2*{-m2}.... + h_i*{-m_i} for all disclosed messages `m_i` and for all undisclosed messages `m_j`.
+        // Moreover -g1 + h1*{-m1} + h2*{-m2}.... + h_i*{-m_i} is public and can be efficiently computed as -(g1 + h1*m1 + h2*m2 +.... * h_i*m_i)
 
         let mut bases_2 = Vec::with_capacity(2 + blindings.len());
         let mut scalars_2 = Vec::with_capacity(2 + blindings.len());
@@ -308,7 +324,7 @@ where
 
         // Verify the randomized signature
         if !E::product_of_pairings(&[
-            (E::G1Prepared::from(self.A_prime), E::G2Prepared::from(pk.w)),
+            (E::G1Prepared::from(self.A_prime), E::G2Prepared::from(pk.0)),
             (
                 E::G1Prepared::from(-self.A_bar),
                 E::G2Prepared::from(params.g2),
@@ -325,11 +341,15 @@ where
         let mut A_bar_minus_d = self.A_bar.into_projective();
         A_bar_minus_d -= self.d.into_projective();
         let A_bar_minus_d = A_bar_minus_d.into_affine();
-        if !self
+        match self
             .sc_resp_1
-            .is_valid(&bases_1, &A_bar_minus_d, &self.T1, challenge)?
+            .is_valid(&bases_1, &A_bar_minus_d, &self.T1, challenge)
         {
-            return Err(BBSPlusError::FirstSchnorrVerificationFailed);
+            Ok(()) => (),
+            Err(SchnorrError::InvalidResponse) => {
+                return Err(BBSPlusError::FirstSchnorrVerificationFailed)
+            }
+            Err(other) => return Err(BBSPlusError::SchnorrError(other)),
         }
 
         // Verify the 2nd Schnorr proof
@@ -351,14 +371,15 @@ where
                 bases_2.push(params.h[i]);
             }
         }
-        // pr = g1 * h1^-m1 * h2^-m2.... = (g1 * h1^m1 * h2^m2....)^-1 for all disclosed messages m_i
+        // pr = -g1 + h1*-m1 + h2*-m2.... = -(g1 + h1*m1 * h2*m2....) for all disclosed messages m_i
         let pr = -VariableBaseMSM::multi_scalar_mul(&bases_disclosed, &exponents);
         let pr = pr.into_affine();
-        if !self
-            .sc_resp_2
-            .is_valid(&bases_2, &pr, &self.T2, challenge)?
-        {
-            return Err(BBSPlusError::SecondSchnorrVerificationFailed);
+        match self.sc_resp_2.is_valid(&bases_2, &pr, &self.T2, challenge) {
+            Ok(()) => (),
+            Err(SchnorrError::InvalidResponse) => {
+                return Err(BBSPlusError::SecondSchnorrVerificationFailed)
+            }
+            Err(other) => return Err(BBSPlusError::SchnorrError(other)),
         }
 
         Ok(())
@@ -415,10 +436,12 @@ mod serialization {
             self.A_prime.serialize(&mut writer)?;
             self.A_bar.serialize(&mut writer)?;
             self.d.serialize(&mut writer)?;
-            self.sc_comm_1.serialize(&mut writer)?;
+            // self.sc_comm_1.serialize(&mut writer)?;
+            ark_serialize::CanonicalSerialize::serialize(&self.sc_comm_1, &mut writer)?;
             self.sc_wits_1[0].serialize(&mut writer)?;
             self.sc_wits_1[1].serialize(&mut writer)?;
-            self.sc_comm_2.serialize(&mut writer)?;
+            // self.sc_comm_2.serialize(&mut writer)?;
+            ark_serialize::CanonicalSerialize::serialize(&self.sc_comm_2, &mut writer)?;
             self.sc_wits_2.serialize(&mut writer)
         }
 
@@ -475,12 +498,14 @@ mod serialization {
             let A_prime = E::G1Affine::deserialize(&mut reader)?;
             let A_bar = E::G1Affine::deserialize(&mut reader)?;
             let d = E::G1Affine::deserialize(&mut reader)?;
-            let sc_comm_1 = <SchnorrCommitment<E::G1Affine>>::deserialize(&mut reader)?;
+            // let sc_comm_1 = <SchnorrCommitment<E::G1Affine>>::deserialize(&mut reader)?;
+            let sc_comm_1 = ark_serialize::CanonicalDeserialize::deserialize(&mut reader)?;
             let sc_wits_1 = [
                 E::Fr::deserialize(&mut reader)?,
                 E::Fr::deserialize(&mut reader)?,
             ];
-            let sc_comm_2 = <SchnorrCommitment<E::G1Affine>>::deserialize(&mut reader)?;
+            // let sc_comm_2 = <SchnorrCommitment<E::G1Affine>>::deserialize(&mut reader)?;
+            let sc_comm_2 = ark_serialize::CanonicalDeserialize::deserialize(&mut reader)?;
             let sc_wits_2 = <Vec<E::Fr>>::deserialize(&mut reader)?;
             Ok(Self {
                 A_prime,
@@ -553,7 +578,7 @@ mod tests {
         UniformRand,
     };
     use blake2::Blake2b;
-    use schnorr::compute_random_oracle_challenge;
+    use schnorr_pok::compute_random_oracle_challenge;
     use std::time::{Duration, Instant};
 
     type Fr = <Bls12_381 as PairingEngine>::Fr;
@@ -572,7 +597,7 @@ mod tests {
             .map(|_| Fr::rand(rng))
             .collect();
         let params = SignatureParamsG1::<Bls12_381>::generate_using_rng(rng, message_count);
-        let keypair = KeypairG2::<Bls12_381>::generate(rng, &params);
+        let keypair = KeypairG2::<Bls12_381>::generate_using_rng(rng, &params);
         let sig =
             SignatureG1::<Bls12_381>::new(rng, &messages, &keypair.secret_key, &params).unwrap();
         (messages, params, keypair, sig)
@@ -609,7 +634,13 @@ mod tests {
         proof_create_duration += start.elapsed();
 
         // Protocol can be serialized
-        test_serialization!(PoKOfSignatureG1Protocol, pok);
+        test_serialization!(PoKOfSignatureG1Protocol<Bls12_381>, pok);
+        let pok_json = serde_json::to_string(&pok).unwrap();
+        println!("serde pok ser={:?}", pok_json);
+        let pok_json_deser =
+            serde_json::from_str::<PoKOfSignatureG1Protocol<Bls12_381>>(&pok_json).unwrap();
+        println!("serde pok deser={:?}", pok_json_deser);
+        assert_eq!(pok_json_deser, pok);
 
         let mut chal_bytes_prover = vec![];
         pok.challenge_contribution(&revealed_msgs, &params, &mut chal_bytes_prover)
@@ -640,7 +671,7 @@ mod tests {
         proof_verif_duration += start.elapsed();
 
         // Proof can be serialized
-        test_serialization!(PoKOfSignatureG1Proof, proof);
+        test_serialization!(PoKOfSignatureG1Proof<Bls12_381>, proof);
 
         println!(
             "Time to create proof with message size {} and revealing {} messages is {:?}",
@@ -668,8 +699,8 @@ mod tests {
             SignatureParamsG1::<Bls12_381>::new::<Blake2b>("test".as_bytes(), message_1_count);
         let params_2 =
             SignatureParamsG1::<Bls12_381>::new::<Blake2b>("test-1".as_bytes(), message_2_count);
-        let keypair_1 = KeypairG2::<Bls12_381>::generate(&mut rng, &params_1);
-        let keypair_2 = KeypairG2::<Bls12_381>::generate(&mut rng, &params_2);
+        let keypair_1 = KeypairG2::<Bls12_381>::generate_using_rng(&mut rng, &params_1);
+        let keypair_2 = KeypairG2::<Bls12_381>::generate_using_rng(&mut rng, &params_2);
 
         let mut messages_1: Vec<Fr> = (0..message_1_count - 1)
             .into_iter()
